@@ -32,7 +32,22 @@ ignored_files = {
   "rngparser.c": "not yet integrated",
   "rngparser.h": "not yet integrated",
   "elfgcchack.h": "not a normal header",
+  "testHTML.c": "test tool",
+  "testReader.c": "test tool",
+  "testSchemas.c": "test tool",
+  "testXPath.c": "test tool",
+  "testAutomata.c": "test tool",
+  "testModule.c": "test tool",
+  "testRegexp.c": "test tool",
+  "testThreads.c": "test tool",
+  "testC14N.c": "test tool",
+  "testRelax.c": "test tool",
+  "testThreadsWin32.c": "test tool",
+  "testSAX.c": "test tool",
+  "testURI.c": "test tool",
   "testapi.c": "generated regression tests",
+  "runtest.c": "regression tests program",
+  "runsuite.c": "regression tests program",
   "tst.c": "not part of the library",
   "testdso.c": "test for dynamid shared libraries",
 }
@@ -958,6 +973,14 @@ class CParser:
 	    elif token[0] == 'preproc':
 		token = self.parsePreproc(token)
 		continue
+	    elif token[0] == "name" and token[1] == "__const":
+	        token = ("name", "const")
+		return token
+	    elif token[0] == "name" and token[1] == "__attribute":
+		token = self.lexer.token()
+		while token != None and token[1] != ";":
+		    token = self.lexer.token()
+		return token
 	    elif token[0] == "name" and ignored_words.has_key(token[1]):
 	        (n, info) = ignored_words[token[1]]
 		i = 0
@@ -1193,7 +1216,9 @@ class CParser:
 	    return token
 
 	while token[0] == "name" and ( 
-	      token[1] == "const" or token[1] == "unsigned"):
+	      token[1] == "const" or \
+	      token[1] == "unsigned" or \
+	      token[1] == "signed"):
 	    if self.type == "":
 	        self.type = token[1]
 	    else:
