@@ -825,7 +825,7 @@ XMLPUBFUN int XMLCALL
  */
 #ifdef LIBXML_SAX1_ENABLED
 XMLPUBFUN xmlDocPtr XMLCALL	
-		xmlParseDoc		(xmlChar *cur);
+		xmlParseDoc		(const xmlChar *cur);
 XMLPUBFUN xmlDocPtr XMLCALL	
 		xmlParseFile		(const char *filename);
 XMLPUBFUN xmlDocPtr XMLCALL	
@@ -836,10 +836,8 @@ XMLPUBFUN int XMLCALL
 		xmlSubstituteEntitiesDefault(int val);
 XMLPUBFUN int XMLCALL		
 		xmlKeepBlanksDefault	(int val);
-#ifdef LIBXML_PUSH_ENABLED
 XMLPUBFUN void XMLCALL		
 		xmlStopParser		(xmlParserCtxtPtr ctxt);
-#endif /* LIBXML_PUSH_ENABLED */
 XMLPUBFUN int XMLCALL		
 		xmlPedanticParserDefault(int val);
 XMLPUBFUN int XMLCALL		
@@ -877,7 +875,7 @@ XMLPUBFUN int XMLCALL
 					 int size);
 XMLPUBFUN xmlDocPtr XMLCALL	
 		xmlSAXParseDoc		(xmlSAXHandlerPtr sax,
-					 xmlChar *cur,
+					 const xmlChar *cur,
 					 int recovery);
 XMLPUBFUN xmlDocPtr XMLCALL	
 		xmlSAXParseMemory	(xmlSAXHandlerPtr sax,
